@@ -1,22 +1,140 @@
+// Copyright 2014-2016 bluss and ndarray developers.
+//
+// Licensed under the Apache License, Version 2.0 <LICENSE-APACHE or
+// http://www.apache.org/licenses/LICENSE-2.0> or the MIT license
+// <LICENSE-MIT or http://opensource.org/licenses/MIT>, at your
+// option. This file may not be copied, modified, or distributed
+// except according to those terms.
+use std::cmp::Ordering;
+use std::fmt::Debug;
 use std::slice;
 
 use super::{Si, Ix, Ixs};
 use super::zipsl;
+use error::{from_kind, ErrorKind, ShapeError};
 
 /// Calculate offset from `Ix` stride converting sign properly
 #[inline]
-pub fn stride_offset(n: Ix, stride: Ix) -> isize
-{
+pub fn stride_offset(n: Ix, stride: Ix) -> isize {
     (n as isize) * ((stride as Ixs) as isize)
 }
 
-/// Trait for the shape and index types of arrays.
+/// Check whether `stride` is strictly positive
+#[inline]
+fn stride_is_positive(stride: Ix) -> bool {
+    (stride as Ixs) > 0
+}
+
+/// Return the axis ordering corresponding to the fastest variation
+///
+/// Assumes that no stride value appears twice. This cannot yield the correct
+/// result the strides are not positive.
+fn fastest_varying_order<D: Dimension>(strides: &D) -> D {
+    let mut sorted = strides.clone();
+    sorted.slice_mut().sort();
+    let mut res = strides.clone();
+    for (index, &val) in strides.slice().iter().enumerate() {
+        let sorted_ind = sorted.slice()
+                               .iter()
+                               .position(|&x| x == val)
+                               .unwrap(); // cannot panic by construction
+        res.slice_mut()[sorted_ind] = index;
+    }
+    res
+}
+
+/// Check whether the given `dim` and `stride` lead to overlapping indices
+///
+/// There is overlap if, when iterating through the dimensions in the order
+/// of maximum variation, the current stride is inferior to the sum of all
+/// preceding strides multiplied by their corresponding dimensions.
+///
+/// The current implementation assumes strides to be positive
+pub fn dim_stride_overlap<D: Dimension>(dim: &D, strides: &D) -> bool {
+    let order = fastest_varying_order(strides);
+
+    let mut prev_offset = 1;
+    for &index in order.slice().iter() {
+        let s = strides.slice()[index];
+        if (s as isize) < prev_offset {
+            return true;
+        }
+        prev_offset = stride_offset(dim.slice()[index], s);
+    }
+    false
+}
+
+/// Check whether the given dimension and strides are memory safe
+/// to index the provided slice.
+///
+/// To be safe, no stride may be negative, and the offset corresponding
+/// to the last element of each dimension should be smaller than the length
+/// of the slice. Also, the strides should not allow a same element to be
+/// referenced by two different index.
+pub fn can_index_slice<A, D: Dimension>(data: &[A], dim: &D, strides: &D)
+    -> Result<(), ShapeError>
+{
+    if strides.slice().iter().cloned().all(stride_is_positive) {
+        if dim.size_checked().is_none() {
+            return Err(from_kind(ErrorKind::OutOfBounds));
+        }
+        let mut last_index = dim.clone();
+        for mut index in last_index.slice_mut().iter_mut() {
+            *index -= 1;
+        }
+        if let Some(offset) = stride_offset_checked_arithmetic(dim,
+                                                               strides,
+                                                               &last_index)
+        {
+            // offset is guaranteed to be positive so no issue converting
+            // to usize here
+            if (offset as usize) >= data.len() {
+                return Err(from_kind(ErrorKind::OutOfBounds));
+            }
+            if dim_stride_overlap(dim, strides) {
+                return Err(from_kind(ErrorKind::Unsupported));
+            }
+        } else {
+            return Err(from_kind(ErrorKind::OutOfBounds));
+        }
+        Ok(())
+    } else {
+        return Err(from_kind(ErrorKind::Unsupported));
+    }
+}
+
+/// Return stride offset for this dimension and index.
+///
+/// Return None if the indices are out of bounds, or the calculation would wrap
+/// around.
+fn stride_offset_checked_arithmetic<D>(dim: &D, strides: &D, index: &D)
+    -> Option<isize>
+    where D: Dimension
+{
+    let mut offset = 0;
+    for ((&d, &i), &s) in zipsl(zipsl(dim.slice(), index.slice()), strides.slice()) {
+        if i >= d {
+            return None;
+        }
+
+        if let Some(offset_) = (i as isize)
+                                   .checked_mul((s as Ixs) as isize)
+                                   .and_then(|x| x.checked_add(offset)) {
+            offset = offset_;
+        } else {
+            return None;
+        }
+    }
+    Some(offset)
+}
+
+/// Array shape and index trait.
 ///
 /// `unsafe` because of the assumptions in the default methods.
 ///
-/// ***Don't implement this trait, it's internal to the crate and will
-/// evolve at will.***
-pub unsafe trait Dimension : Clone + Eq {
+/// ***Don't implement or call methods in this trait, its interface is internal
+/// to the crate and will evolve at will.***
+pub unsafe trait Dimension : Clone + Eq + Debug {
     /// `SliceArg` is the type which is used to specify slicing for this
     /// dimension.
     ///
@@ -32,23 +150,34 @@ pub unsafe trait Dimension : Clone + Eq {
     /// The easiest way to create a `&SliceArg` is using the macro
     /// [`s![]`](macro.s!.html).
     type SliceArg: ?Sized + AsRef<[Si]>;
+    #[doc(hidden)]
     fn ndim(&self) -> usize;
+    #[doc(hidden)]
     fn slice(&self) -> &[Ix] {
         unsafe {
             slice::from_raw_parts(self as *const _ as *const Ix, self.ndim())
         }
     }
 
+    #[doc(hidden)]
     fn slice_mut(&mut self) -> &mut [Ix] {
         unsafe {
             slice::from_raw_parts_mut(self as *mut _ as *mut Ix, self.ndim())
         }
     }
 
+    #[doc(hidden)]
     fn size(&self) -> usize {
         self.slice().iter().fold(1, |s, &a| s * a as usize)
     }
 
+    #[doc(hidden)]
+    /// Compute the size while checking for overflow
+    fn size_checked(&self) -> Option<usize> {
+        self.slice().iter().fold(Some(1), |s, &a| s.and_then(|s_| s_.checked_mul(a)))
+    }
+
+    #[doc(hidden)]
     fn default_strides(&self) -> Self {
         // Compute default array strides
         // Shape (a, b, c) => Give strides (b * c, c, 1)
@@ -69,12 +198,33 @@ pub unsafe trait Dimension : Clone + Eq {
         strides
     }
 
+    #[doc(hidden)]
+    fn fortran_strides(&self) -> Self {
+        // Compute fortran array strides
+        // Shape (a, b, c) => Give strides (1, a, a * b)
+        let mut strides = self.clone();
+        {
+            let mut it = strides.slice_mut().iter_mut();
+            // Set first element to 1
+            for rs in it.by_ref() {
+                *rs = 1;
+                break;
+            }
+            let mut cum_prod = 1;
+            for (rs, dim) in it.zip(self.slice().iter()) {
+                cum_prod *= *dim;
+                *rs = cum_prod;
+            }
+        }
+        strides
+    }
+
+    #[doc(hidden)]
     #[inline]
-    fn first_index(&self) -> Option<Self>
-    {
+    fn first_index(&self) -> Option<Self> {
         for ax in self.slice().iter() {
             if *ax == 0 {
-                return None
+                return None;
             }
         }
         let mut index = self.clone();
@@ -84,6 +234,7 @@ pub unsafe trait Dimension : Clone + Eq {
         Some(index)
     }
 
+    #[doc(hidden)]
     /// Iteration -- Use self as size, and return next index after `index`
     /// or None if there are no more.
     // FIXME: use &Self for index or even &mut?
@@ -91,8 +242,7 @@ pub unsafe trait Dimension : Clone + Eq {
     fn next_for(&self, index: Self) -> Option<Self> {
         let mut index = index;
         let mut done = false;
-        for (&dim, ix) in zipsl(self.slice(), index.slice_mut()).rev()
-        {
+        for (&dim, ix) in zipsl(self.slice(), index.slice_mut()).rev() {
             *ix += 1;
             if *ix == dim {
                 *ix = 0;
@@ -103,12 +253,14 @@ pub unsafe trait Dimension : Clone + Eq {
         }
         if done {
             Some(index)
-        } else { None }
+        } else {
+            None
+        }
     }
 
+    #[doc(hidden)]
     /// Return stride offset for index.
-    fn stride_offset(index: &Self, strides: &Self) -> isize
-    {
+    fn stride_offset(index: &Self, strides: &Self) -> isize {
         let mut offset = 0;
         for (&i, &s) in zipsl(index.slice(), strides.slice()) {
             offset += stride_offset(i, s);
@@ -116,14 +268,12 @@ pub unsafe trait Dimension : Clone + Eq {
         offset
     }
 
+    #[doc(hidden)]
     /// Return stride offset for this dimension and index.
-    fn stride_offset_checked(&self, strides: &Self, index: &Self) -> Option<isize>
-    {
+    fn stride_offset_checked(&self, strides: &Self, index: &Self) -> Option<isize> {
         let mut offset = 0;
-        for ((&d, &i), &s) in zipsl(zipsl(self.slice(),
-                                          index.slice()),
-                                    strides.slice())
-        {
+        for ((&d, &i), &s) in zipsl(zipsl(self.slice(), index.slice()),
+                                    strides.slice()) {
             if i >= d {
                 return None;
             }
@@ -132,19 +282,17 @@ pub unsafe trait Dimension : Clone + Eq {
         Some(offset)
     }
 
+    #[doc(hidden)]
     /// Modify dimension, strides and return data pointer offset
     ///
     /// **Panics** if `slices` does not correspond to the number of axes,
     /// if any stride is 0, or if any index is out of bounds.
-    fn do_slices(dim: &mut Self, strides: &mut Self, slices: &Self::SliceArg) -> isize
-    {
+    fn do_slices(dim: &mut Self, strides: &mut Self, slices: &Self::SliceArg) -> isize {
         let slices = slices.as_ref();
         let mut offset = 0;
         assert!(slices.len() == dim.slice().len());
-        for ((dr, sr), &slc) in zipsl(zipsl(dim.slice_mut(),
-                                            strides.slice_mut()),
-                                      slices)
-        {
+        for ((dr, sr), &slc) in zipsl(zipsl(dim.slice_mut(), strides.slice_mut()),
+                                      slices) {
             let m = *dr;
             let mi = m as Ixs;
             let Si(b1, opt_e1, s1) = slc;
@@ -186,12 +334,59 @@ pub unsafe trait Dimension : Clone + Eq {
         }
         offset
     }
+
+}
+
+/// Implementation-specific extensions to `Dimension`
+pub trait DimensionExt {
+// note: many extensions go in the main trait if they need to be special-
+// cased per dimension
+    /// Get the dimension at `axis`.
+    ///
+    /// *Panics* if `axis` is out of bounds.
+    #[inline]
+    fn axis(&self, axis: Axis) -> Ix;
+
+    /// Set the dimension at `axis`.
+    ///
+    /// *Panics* if `axis` is out of bounds.
+    #[inline]
+    fn set_axis(&mut self, axis: Axis, value: Ix);
+}
+
+impl<D> DimensionExt for D
+    where D: Dimension
+{
+    #[inline]
+    fn axis(&self, axis: Axis) -> Ix {
+        self.slice()[axis.axis()]
+    }
+
+    #[inline]
+    fn set_axis(&mut self, axis: Axis, value: Ix) {
+        self.slice_mut()[axis.axis()] = value;
+    }
+}
+
+impl<'a> DimensionExt for [Ix]
+{
+    #[inline]
+    fn axis(&self, axis: Axis) -> Ix {
+        self[axis.axis()]
+    }
+
+    #[inline]
+    fn set_axis(&mut self, axis: Axis, value: Ix) {
+        self[axis.axis()] = value;
+    }
 }
 
 fn abs_index(len: Ixs, index: Ixs) -> Ix {
     if index < 0 {
         (len + index) as Ix
-    } else { index as Ix }
+    } else {
+        index as Ix
+    }
 }
 
 /// Collapse axis `axis` and shift so that only subarray `index` is
@@ -200,8 +395,7 @@ fn abs_index(len: Ixs, index: Ixs) -> Ix {
 /// **Panics** if `index` is larger than the size of the axis
 // FIXME: Move to Dimension trait
 pub fn do_sub<A, D: Dimension>(dims: &mut D, ptr: &mut *mut A, strides: &D,
-                               axis: usize, index: Ix)
-{
+                               axis: usize, index: Ix) {
     let dim = dims.slice()[axis];
     let stride = strides.slice()[axis];
     assert!(index < dim);
@@ -228,6 +422,8 @@ unsafe impl Dimension for Ix {
     fn ndim(&self) -> usize { 1 }
     #[inline]
     fn size(&self) -> usize { *self as usize }
+    #[inline]
+    fn size_checked(&self) -> Option<usize> { Some(*self as usize) }
 
     #[inline]
     fn default_strides(&self) -> Self { 1 }
@@ -236,27 +432,29 @@ unsafe impl Dimension for Ix {
     fn first_index(&self) -> Option<Ix> {
         if *self != 0 {
             Some(0)
-        } else { None }
+        } else {
+            None
+        }
     }
     #[inline]
     fn next_for(&self, mut index: Ix) -> Option<Ix> {
         index += 1;
         if index < *self {
             Some(index)
-        } else { None }
+        } else {
+            None
+        }
     }
 
     /// Self is an index, return the stride offset
     #[inline]
-    fn stride_offset(index: &Ix, stride: &Ix) -> isize
-    {
+    fn stride_offset(index: &Ix, stride: &Ix) -> isize {
         stride_offset(*index, *stride)
     }
 
     /// Return stride offset for this dimension and index.
     #[inline]
-    fn stride_offset_checked(&self, stride: &Ix, index: &Ix) -> Option<isize>
-    {
+    fn stride_offset_checked(&self, stride: &Ix, index: &Ix) -> Option<isize> {
         if *index < *self {
             Some(stride_offset(*index, *stride))
         } else {
@@ -269,8 +467,16 @@ unsafe impl Dimension for (Ix, Ix) {
     type SliceArg = [Si; 2];
     #[inline]
     fn ndim(&self) -> usize { 2 }
+
     #[inline]
     fn size(&self) -> usize { let (m, n) = *self; m as usize * n as usize }
+
+    #[inline]
+    fn size_checked(&self) -> Option<usize> {
+        let (m, n) = *self;
+        (m as usize).checked_mul(n as usize)
+    }
+
     #[inline]
     fn default_strides(&self) -> Self {
         // Compute default array strides
@@ -283,7 +489,9 @@ unsafe impl Dimension for (Ix, Ix) {
         let (m, n) = *self;
         if m != 0 && n != 0 {
             Some((0, 0))
-        } else { None }
+        } else {
+            None
+        }
     }
     #[inline]
     fn next_for(&self, index: (Ix, Ix)) -> Option<(Ix, Ix)> {
@@ -302,8 +510,7 @@ unsafe impl Dimension for (Ix, Ix) {
 
     /// Self is an index, return the stride offset
     #[inline]
-    fn stride_offset(index: &(Ix, Ix), strides: &(Ix, Ix)) -> isize
-    {
+    fn stride_offset(index: &(Ix, Ix), strides: &(Ix, Ix)) -> isize {
         let (i, j) = *index;
         let (s, t) = *strides;
         stride_offset(i, s) + stride_offset(j, t)
@@ -351,8 +558,7 @@ unsafe impl Dimension for (Ix, Ix, Ix) {
 
     /// Self is an index, return the stride offset
     #[inline]
-    fn stride_offset(index: &(Ix, Ix, Ix), strides: &(Ix, Ix, Ix)) -> isize
-    {
+    fn stride_offset(index: &(Ix, Ix, Ix), strides: &(Ix, Ix, Ix)) -> isize {
         let (i, j, k) = *index;
         let (s, t, u) = *strides;
         stride_offset(i, s) + stride_offset(j, t) + stride_offset(k, u)
@@ -389,11 +595,13 @@ unsafe impl Dimension for Vec<Ix>
     fn slice_mut(&mut self) -> &mut [Ix] { self }
 }
 
-/// Helper trait to define a larger-than relation for array shapes:
+/// Array shape with a next smaller dimension.
+///
+/// `RemoveAxis` defines a larger-than relation for array shapes:
 /// removing one axis from *Self* gives smaller dimension *Smaller*.
 pub trait RemoveAxis : Dimension {
     type Smaller: Dimension;
-    fn remove_axis(&self, axis: usize) -> Self::Smaller;
+    fn remove_axis(&self, axis: Axis) -> Self::Smaller;
 }
 
 macro_rules! impl_shrink(
@@ -403,12 +611,12 @@ impl RemoveAxis for ($from $(,$more)*)
     type Smaller = ($($more),*);
     #[allow(unused_parens)]
     #[inline]
-    fn remove_axis(&self, axis: usize) -> ($($more),*) {
+    fn remove_axis(&self, axis: Axis) -> ($($more),*) {
         let mut tup = ($(0 as $more),*);
         {
             let mut it = tup.slice_mut().iter_mut();
             for (i, &d) in self.slice().iter().enumerate() {
-                if i == axis {
+                if i == axis.axis() {
                     continue;
                 }
                 for rr in it.by_ref() {
@@ -434,18 +642,16 @@ macro_rules! impl_shrink_recursive(
 // 12 is the maximum number for having the Eq trait from libstd
 impl_shrink_recursive!(Ix, Ix, Ix, Ix, Ix, Ix, Ix, Ix, Ix, Ix, Ix, Ix,);
 
-impl RemoveAxis for Vec<Ix>
-{
+impl RemoveAxis for Vec<Ix> {
     type Smaller = Vec<Ix>;
-    fn remove_axis(&self, axis: usize) -> Vec<Ix>
-    {
+    fn remove_axis(&self, axis: Axis) -> Vec<Ix> {
         let mut res = self.clone();
-        res.remove(axis);
+        res.remove(axis.axis());
         res
     }
 }
 
-/// A tuple or fixed size array that can be used to index an array.
+/// Tuple or fixed size arrays that can be used to index an array.
 ///
 /// ```
 /// use ndarray::arr2;
@@ -458,13 +664,15 @@ impl RemoveAxis for Vec<Ix>
 ///
 /// **Note** the blanket implementation that's not visible in rustdoc:
 /// `impl<D> NdIndex for D where D: Dimension { ... }`
-pub unsafe trait NdIndex {
+pub unsafe trait NdIndex : Debug {
     type Dim: Dimension;
     #[doc(hidden)]
     fn index_checked(&self, dim: &Self::Dim, strides: &Self::Dim) -> Option<isize>;
 }
 
-unsafe impl<D> NdIndex for D where D: Dimension {
+unsafe impl<D> NdIndex for D
+    where D: Dimension
+{
     type Dim = D;
     fn index_checked(&self, dim: &Self::Dim, strides: &Self::Dim) -> Option<isize> {
         dim.stride_offset_checked(strides, self)
@@ -518,9 +726,7 @@ unsafe impl<'a> NdIndex for &'a [Ix] {
     type Dim = Vec<Ix>;
     fn index_checked(&self, dim: &Self::Dim, strides: &Self::Dim) -> Option<isize> {
         let mut offset = 0;
-        for ((&d, &i), &s) in zipsl(zipsl(&dim[..], &self[..]),
-                                    strides.slice())
-        {
+        for ((&d, &i), &s) in zipsl(zipsl(&dim[..], &self[..]), strides.slice()) {
             if i >= d {
                 return None;
             }
@@ -529,3 +735,80 @@ unsafe impl<'a> NdIndex for &'a [Ix] {
         Some(offset)
     }
 }
+
+// NOTE: These tests are not compiled & tested
+#[cfg(test)]
+mod test {
+    use super::Dimension;
+    use error::StrideError;
+
+    #[test]
+    fn fastest_varying_order() {
+        let strides = (2, 8, 4, 1);
+        let order = super::fastest_varying_order(&strides);
+        assert_eq!(order.slice(), &[3, 0, 2, 1]);
+    }
+
+    #[test]
+    fn slice_indexing_uncommon_strides() {
+        let v: Vec<_> = (0..12).collect();
+        let dim = (2, 3, 2);
+        let strides = (1, 2, 6);
+        assert!(super::can_index_slice(&v, &dim, &strides).is_ok());
+
+        let strides = (2, 4, 12);
+        assert_eq!(super::can_index_slice(&v, &dim, &strides),
+                   Err(StrideError::OutOfBounds));
+    }
+
+    #[test]
+    fn overlapping_strides_dim() {
+        let dim = (2, 3, 2);
+        let strides = (5, 2, 1);
+        assert!(super::dim_stride_overlap(&dim, &strides));
+        let strides = (6, 2, 1);
+        assert!(!super::dim_stride_overlap(&dim, &strides));
+        let strides = (6, 0, 1);
+        assert!(super::dim_stride_overlap(&dim, &strides));
+    }
+}
+
+/// An axis index.
+///
+/// An axis one of an array’s “dimensions”; an *n*-dimensional array has *n* axes.
+/// Axis *0* is the array’s outermost axis and *n*-1 is the innermost.
+///
+/// All array axis arguments use this type to make the code easier to write
+/// correctly and easier to understand.
+#[derive(Copy, Eq, Ord, Hash, Debug)]
+pub struct Axis(pub usize);
+
+impl Axis {
+    #[inline(always)]
+    pub fn axis(&self) -> usize { self.0 }
+}
+
+macro_rules! clone_from_copy {
+    ($typename:ident) => {
+        impl Clone for $typename {
+            #[inline]
+            fn clone(&self) -> Self { *self }
+        }
+    }
+}
+
+macro_rules! derive_cmp {
+    ($traitname:ident for $typename:ident, $method:ident -> $ret:ty) => {
+        impl $traitname for $typename {
+            #[inline(always)]
+            fn $method(&self, rhs: &Self) -> $ret {
+                (self.0).$method(&rhs.0)
+            }
+        }
+    }
+}
+
+derive_cmp!{PartialEq for Axis, eq -> bool}
+derive_cmp!{PartialOrd for Axis, partial_cmp -> Option<Ordering>}
+clone_from_copy!{Axis}
+
